@@ -11,6 +11,10 @@ module ClassyPrelude
       -- ** Semigroup
     , Semigroup (..)
     , WrappedMonoid
+      -- ** Functor
+    , module Data.Functor
+      -- ** Applicative
+    , module Control.Applicative
       -- ** Monad
     , module Control.Monad
     , whenM
@@ -26,6 +30,7 @@ module ClassyPrelude
     , orElseSTM
     , checkSTM
     , module Data.IORef.Lifted
+    , module Data.Mutable
       -- ** Primitive (exported since 0.9.4)
     , PrimMonad
     , PrimState
@@ -95,6 +100,7 @@ module ClassyPrelude
     , foldl'
     , foldr
     , foldM
+    , elem
     --, split
     , readMay
     , intercalate
@@ -134,6 +140,10 @@ module ClassyPrelude
     , fpFromText
     , fpToTextWarn
     , fpToTextEx
+      -- ** Difference lists
+    , DList
+    , asDList
+    , applyDList
       -- ** Exceptions
     , module Control.Exception.Enclosed
     , MonadThrow (throwM)
@@ -160,6 +170,8 @@ module ClassyPrelude
     ) where
 
 import qualified Prelude
+import Control.Applicative ((<**>),liftA,liftA2,liftA3)
+import Data.Functor
 import Control.Exception (assert)
 import Control.Exception.Enclosed
 import Control.Monad (when, unless, void, liftM, ap, forever, join, replicateM_, guard, MonadPlus (..), (=<<), (>=>), (<=<), liftM2, liftM3, liftM4, liftM5)
@@ -168,6 +180,7 @@ import Control.Concurrent.Chan.Lifted
 import Control.Concurrent.STM hiding (atomically, always, alwaysSucceeds, retry, orElse, check)
 import qualified Control.Concurrent.STM as STM
 import Data.IORef.Lifted
+import Data.Mutable
 import qualified Data.Monoid as Monoid
 import Data.Traversable (Traversable (..), for, forM)
 import Data.Foldable (Foldable)
@@ -178,7 +191,7 @@ import Data.Vector.Instances ()
 import CorePrelude hiding (print, undefined, (<>), catMaybes, first, second)
 import Data.ChunkedZip
 import qualified Data.Char as Char
-import Data.Sequences
+import Data.Sequences hiding (elem)
 import Data.MonoTraversable
 import Data.Containers
 import Data.Builder
@@ -199,7 +212,7 @@ import Data.Time
     , parseTime
     , getCurrentTime
     )
-import System.Locale (defaultTimeLocale)
+import Data.Time.Locale.Compat (defaultTimeLocale)
 
 import qualified Data.Set as Set
 import qualified Data.Map as Map
@@ -216,6 +229,8 @@ import Data.Primitive.Types (Prim)
 import Data.Functor.Identity (Identity (..))
 import Control.Monad.Reader (MonadReader, ask, ReaderT (..), Reader)
 import Data.Bifunctor
+import Data.DList (DList)
+import qualified Data.DList as DList
 
 tshow :: Show a => a -> Text
 tshow = fromList . Prelude.show
@@ -226,14 +241,14 @@ tlshow = fromList . Prelude.show
 -- | Convert a character to lower case.
 --
 -- Character-based case conversion is lossy in comparison to string-based 'Data.MonoTraversable.toLower'.
--- For instance, \'&#x130;\' will be converted to \'i\', instead of \"i&#x307;\".
+-- For instance, &#x130; will be converted to i, instead of i&#x307;.
 charToLower :: Char -> Char
 charToLower = Char.toLower
 
 -- | Convert a character to upper case.
 --
 -- Character-based case conversion is lossy in comparison to string-based 'Data.MonoTraversable.toUpper'.
--- For instance, \'&#xdf;\' won't be converted to \"SS\".
+-- For instance, &#xdf; won't be converted to SS.
 charToUpper :: Char -> Char
 charToUpper = Char.toUpper
 
@@ -293,6 +308,10 @@ concatMap :: (Monoid m, MonoFoldable c) => (Element c -> m) -> c -> m
 concatMap = ofoldMap
 {-# INLINE concatMap #-}
 
+elem :: (MonoFoldableEq c) => Element c -> c -> Bool
+elem = oelem
+{-# INLINE elem #-}
+
 foldMap :: (Monoid m, MonoFoldable c) => (Element c -> m) -> c -> m
 foldMap = ofoldMap
 {-# INLINE foldMap #-}
@@ -334,12 +353,12 @@ infixr 5  ++
 {-# INLINE (++) #-}
 
 infixl 9 \\{-This comment teaches CPP correct behaviour -}
--- | An alias for `difference`.
+-- | An alias for 'difference'.
 (\\) :: SetContainer a => a -> a -> a
 (\\) = difference
 {-# INLINE (\\) #-}
 
--- | An alias for `intersection`.
+-- | An alias for 'intersection'.
 intersect :: SetContainer a => a -> a -> a
 intersect = intersection
 {-# INLINE intersect #-}
@@ -407,14 +426,14 @@ print = liftIO . Prelude.print
 sortWith :: (Ord a, IsSequence c) => (Element c -> a) -> c -> c
 sortWith f = sortBy $ comparing f
 
--- | We define our own @undefined@ which is marked as deprecated. This makes it
--- useful to use during development, but let's you more easily getting
--- notification if you accidentally ship partial code in production.
+-- | We define our own 'undefined' which is marked as deprecated. This makes it
+-- useful to use during development, but lets you more easily get
+-- notifications if you accidentally ship partial code in production.
 --
 -- The classy prelude recommendation for when you need to really have a partial
--- function in production is to use @error@ with a very descriptive message so
+-- function in production is to use 'error' with a very descriptive message so
 -- that, in case an exception is thrown, you get more information than
--- @Prelude.undefined@.
+-- @"Prelude".'Prelude.undefined'@.
 --
 -- Since 0.5.5
 undefined :: a
@@ -451,9 +470,9 @@ fpToString = F.encodeString
 fpFromString :: String -> FilePath
 fpFromString = F.decodeString
 
--- | Translates a FilePath to a Text
--- Warns if there are non-unicode
--- sequences in the file name
+-- | Translates a 'FilePath' to a 'Text'
+--
+-- Warns if there are non-unicode sequences in the file name
 fpToTextWarn :: MonadIO m => FilePath -> m Text
 fpToTextWarn fp = case F.toText fp of
     Right ok -> return ok
@@ -461,23 +480,24 @@ fpToTextWarn fp = case F.toText fp of
         putStrLn $ pack $ "non-unicode filepath: " ++ F.encodeString fp
         return bad
 
--- | Translates a FilePath to a Text
+-- | Translates a 'FilePath' to a 'Text'
+--
 -- Throws an exception if there are non-unicode
 -- sequences in the file name
 --
 -- Use this to assert that you know
--- a filename will translate properly into a Text
+-- a filename will translate properly into a 'Text'.
 -- If you created the filename, this should be the case.
 fpToTextEx :: FilePath -> Text
 fpToTextEx fp = either (const $ error errorMsg) id $ F.toText fp
   where
     errorMsg = "fpToTextEx: non-unicode filepath: " ++ F.encodeString fp
 
--- | Translates a FilePath to a Text
+-- | Translates a 'FilePath' to a 'Text'
 -- This translation is not correct for a (unix) filename
--- which can contain arbitrary (non-unicode) bytes: those bytes will be discarded
+-- which can contain arbitrary (non-unicode) bytes: those bytes will be discarded.
 --
--- This means you cannot translate the Text back to the original file name.
+-- This means you cannot translate the 'Text' back to the original file name.
 --
 -- If you control or otherwise understand the filenames
 -- and believe them to be unicode valid consider using 'fpToTextEx' or 'fpToTextWarn'
@@ -492,8 +512,9 @@ fpFromText = F.fromText
 -- Should eventually exist in mono-foldable and be extended to MonoFoldable
 -- when doing that should re-run the haskell-ordnub benchmarks
 
--- | same behavior as nub, but requires Hashable & Eq and is O(n log n)
--- https://github.com/nh2/haskell-ordnub
+-- | same behavior as 'Data.List.nub', but requires 'Hashable' & 'Eq' and is @O(n log n)@
+--
+-- <https://github.com/nh2/haskell-ordnub>
 hashNub :: (Hashable a, Eq a) => [a] -> [a]
 hashNub = go HashSet.empty
   where
@@ -501,8 +522,9 @@ hashNub = go HashSet.empty
     go s (x:xs) | x `HashSet.member` s = go s xs
                 | otherwise            = x : go (HashSet.insert x s) xs
 
--- | same behavior as nub, but requires Ord and is O(n log n)
--- https://github.com/nh2/haskell-ordnub
+-- | same behavior as 'Data.List.nub', but requires 'Ord' and is @O(n log n)@
+--
+-- <https://github.com/nh2/haskell-ordnub>
 ordNub :: (Ord a) => [a] -> [a]
 ordNub = go Set.empty
   where
@@ -510,8 +532,9 @@ ordNub = go Set.empty
     go s (x:xs) | x `Set.member` s = go s xs
                 | otherwise        = x : go (Set.insert x s) xs
 
--- | same behavior as nubBy, but requires Ord and is O(n log n)
--- https://github.com/nh2/haskell-ordnub
+-- | same behavior as 'Data.List.nubBy', but requires 'Ord' and is @O(n log n)@
+--
+-- <https://github.com/nh2/haskell-ordnub>
 ordNubBy :: (Ord b) => (a -> b) -> (a -> a -> Bool) -> [a] -> [a]
 ordNubBy p f = go Map.empty
   -- When removing duplicates, the first function assigns the input to a bucket,
@@ -558,13 +581,13 @@ checkSTM :: Bool -> STM ()
 checkSTM = STM.check
 {-# INLINE checkSTM #-}
 
--- | Only perform the action if the predicate returns @True@.
+-- | Only perform the action if the predicate returns 'True'.
 --
 -- Since 0.9.2
 whenM :: Monad m => m Bool -> m () -> m ()
 whenM mbool action = mbool >>= flip when action
 
--- | Only perform the action if the predicate returns @False@.
+-- | Only perform the action if the predicate returns 'False'.
 --
 -- Since 0.9.2
 unlessM :: Monad m => m Bool -> m () -> m ()
@@ -573,3 +596,17 @@ unlessM mbool action = mbool >>= flip unless action
 sequence_ :: (Monad m, MonoFoldable mono, Element mono ~ (m a)) => mono -> m ()
 sequence_ = mapM_ (>> return ())
 {-# INLINE sequence_ #-}
+
+-- | Force type to a 'DList'
+--
+-- Since 0.11.0
+asDList :: DList a -> DList a
+asDList = id
+{-# INLINE asDList #-}
+
+-- | Synonym for 'DList.apply'
+--
+-- Since 0.11.0
+applyDList :: DList a -> [a] -> [a]
+applyDList = DList.apply
+{-# INLINE applyDList #-}
